
/**
 * @author Raj
 *
 * Copyright (c) 2025, The University of Melbourne, Australia
 *
 * All rights reserved.a
 */
package au.org.vicnode.mflux.services;

import arc.mf.plugin.PluginService;
import arc.mf.plugin.dtype.StringType;
import arc.xml.XmlDoc;
import arc.xml.XmlDoc.Element;
import arc.xml.XmlDocMaker;
import arc.xml.XmlWriter;
import au.org.vicnode.mflux.plugin.util.Project;
import au.org.vicnode.mflux.plugin.util.Properties;
import au.org.vicnode.mflux.plugin.util.User;
import org.json.JSONArray;
import org.json.JSONObject;
import unimelb.utils.JsonUtils;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;


public class SvcUserRcpUsrRoleSet extends PluginService {

    public static final String SERVICE_NAME = Properties.SERVICE_ROOT + "user.rcpusr.role.set";

    private Interface _defn;

    public SvcUserRcpUsrRoleSet() throws Throwable {
        _defn = new Interface();
    }

    public Access access() {
        return ACCESS_ADMINISTER;
    }

    public Interface definition() {
        return _defn;
    }

    public String description() {
        return "Set the rcpusr role for all users who should have them and remove from users who shouldn't";
    }

    public String name() {
        return SERVICE_NAME;
    }



    public void execute(XmlDoc.Element args, Inputs inputs, Outputs outputs, final XmlWriter w) throws Throwable {
        removeRCPUsrRole();
        Collection<XmlDoc.Element> response = getRcpUsers();
        //w.add("rcpusers",response);
        for (XmlDoc.Element element : response) {
            String path = element.value("path");
            String rcao = element.value("rcao");
            Collection<String> users = element.values("users-with-manage-resources-permission/user");
            users.add(rcao);
            String projectRole = Project.actualRoleName(path, Project.ProjectRoleType.RCPUSR);
            for (String userEmail : users) {
                String domain = "local";
                if (userEmail.contains("student") && userEmail.contains("unimelb")){
                    domain = "student";
                } else if (userEmail.contains("unimelb")) {
                    domain = "unimelb";
                }
                if (userEmail.contains("florey")){
                    domain = null;
                }
                XmlDoc.Element currentUser = null;
                String user = null;
                try {
                    Collection<XmlDoc.Element> userList = User.findUser(executor(), domain, userEmail);
                    currentUser = userList.stream().findFirst().get();
                    user = currentUser.value("@user");
                    domain = currentUser.value("@domain");
                }catch (Throwable e){
                    w.add("user", userEmail);
                    w.add("project", path);
                    w.add("role", projectRole);
                    w.add("error", "User not found");
                    continue;
                }
                //w.add("user",user);
                //w.add("domain",domain);

                try {
                    User.grantRevokeRole(executor(), domain + ":" + user, projectRole, true);
                }catch (Throwable E){
                    w.add("user", userEmail);
                    w.add("project", path);
                    w.add("role", projectRole);
                    w.add("error", "Could not grant user role");
                }
            }
            checkIfThreadTaskAborted();

        }
    }

    @Override
    public boolean canBeAborted() {
        return true;
    }

    public void removeRCPUsrRole() throws Throwable {
        XmlDoc.Element projectElement = executor().execute(Properties.SERVICE_ROOT + "project.list");
        List<XmlDoc.Element> projects = projectElement.elements("project");
        for (XmlDoc.Element project : projects) {
            String rcprole = Project.actualRoleName(project.value(), Project.ProjectRoleType.RCPUSR);
            Vector<String> users = User.haveRole(executor(), rcprole, true);
            if (users.size() > 0) {
                for (String user : users) {
                    User.grantRevokeRole(executor(),user,rcprole,false);
                }
            }
            checkIfThreadTaskAborted();
        }
    }



    public Collection<XmlDoc.Element> getRcpUsers() throws Throwable {
        //unimelb.rcp.allocation.users.list :product mediaflux
        XmlDocMaker dm = new XmlDocMaker("args");
        dm.add("product", "mediaflux");

        XmlDoc.Element r = executor().execute("unimelb.rcp.allocation.users.list", dm.root());
        Collection<XmlDoc.Element> allocations = r.elements("allocation");

        return allocations;
    }
}

